#ifndef __MCP23S17_H__
#define __MCP23S17_H__



namespace MCP23S17 {

#define SERIAL_DEBUG 0
  
//setup register values for this chip
// BANK=1 mode
/*
const int IODIRA   = 0x00;
const int IPOLA    = 0x01;
const int GPINTENA = 0x02;
const int DEFVALA  = 0x03;
const int INTCONA  = 0x04;
const int IOCON   = 0x05;
const int GPPUA    = 0x06;
const int INTFA    = 0x07;
const int INTCAPA  = 0x08;
const int GPIOA    = 0x09;
const int OLATA    = 0x0A;

const int IODIRB   = 0x10;
const int IPOLB    = 0x11;
const int GPINTENB = 0x12;
const int DEFVALB  = 0x13;
const int INTCONB  = 0x14;
const int IOCONB   = 0x15;
const int GPPUB    = 0x16;
const int INTFB    = 0x17;
const int INTCAPB  = 0x18;
const int GPIOB    = 0x19;
const int OLATB    = 0x1A;
*/

// IOCON.Bank = 0
const int IODIRA   = 0x00;
const int IPOLA    = 0x02;
const int GPINTENA = 0x04;
const int DEFVALA  = 0x06;
const int INTCONA  = 0x08;
const int IOCON   = 0x0A;
const int GPPUA    = 0x0C;
const int INTFA    = 0x0E;
const int INTCAPA  = 0x10;
const int GPIOA    = 0x12;
const int OLATA    = 0x14;

const int IODIRB   = 0x01;
const int IPOLB    = 0x03;
const int GPINTENB = 0x05;
const int DEFVALB  = 0x07;
const int INTCONB  = 0x09;
const int IOCONB   = 0x0B;
const int GPPUB    = 0x0D;
const int INTFB    = 0x0F;
const int INTCAPB  = 0x11;
const int GPIOB    = 0x13;
const int OLATB    = 0x15;

const uint8_t PINMODE_INPUT = 0xFF;
const uint8_t PINMODE_OUTPUT = 0x00;

const uint8_t OPCODE_ADDR = 0b01000000;
const uint8_t OPCODE_A0   = 0b00000010;
const uint8_t OPCODE_A1   = 0b00000100;
const uint8_t OPCODE_A2   = 0b00001000;
const uint8_t OPCODE_READ = 0b00000001;

// "B" relays
const uint8_t PINA[9] = {
  0b00000000,
  0b00000000, // OP 1
  0b00000000, // OP 2
  0b00000000, // OP 3
  0b00000000, // OP 4
  0b00000001, // OP 5
  0b00000111, // OP 6
  0b01001001, // OP 7
  0b01110001, // OP 8
};

// "A" relays"
const uint8_t PINB[9] = {
  0b00000000,
  0b00000001, // OP 1
  0b00000110, // OP 2
  0b00011000, // OP 3
  0b01101000, // OP 4
  0b10000000, // OP 5
  0b00000000, // OP 6
  0b00000000, // OP 7
  0b00000000, // OP 8
};

class MCP23S17 {
public:
  MCP23S17( int8_t SDIO, int8_t CSB, int8_t SCLK)  {
    spi = new SPIBitBang(SDIO, CSB, SCLK);
    //write_register(IOCON, 0b01110010);
    //write_register(0x05 , 0b01110010); // if we were in bank=1 mode
  }
  ~MCP23S17() {
    delete spi;
  }
  void set_pin_mode(uint8_t mode) {
    
    write_register(IODIRA, mode);
    write_register(IODIRB, mode);
  }
  void set_gpio(uint8_t stateA, uint8_t stateB) {
    write_register(GPIOA, stateA);
    write_register(GPIOB, stateB);
    //spi->write16(cmd);
  }

  
  void write_register(uint8_t reg, uint8_t data) {
    uint8_t opcode = OPCODE_ADDR; // | OPCODE_A0 | OPCODE_A1;
    if (SERIAL_DEBUG) {
      Serial.print("opcode ");
      Serial.print(opcode, HEX);
      Serial.print(" reg ");
      Serial.print(reg, HEX);
      Serial.print(" data= ");
      Serial.print(data, HEX);
      Serial.print("\n");
    }
    spi->write_three(opcode, reg, data);
    //spi->write24(cmd, data);
    //spi->write16
    //spi->write16(cmd);
  }

  private:
    SPIBitBang* spi;
}; // end class
} // end namespace
#endif 
