/*
 * Firmware for the "RF Multiplexer 8ch 1GHz" open hardware RF-multiplexer
 * project hosted at: https://ohwr.org/project/rf-mux-8ch
 * 
 * 
 *  This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "spibitbang.h"
#include "rf_mux.h"
#include "httpd.h"

// IDC-carrier, version0 pins:
// CS   A3
// SCK  A0
// SI   A1
// SO   A2

// pin-definitions, for the MKR ZERO on the carrier board (2018.11 version)
#define SDIO_PIN A1   // old:3
#define SCLK_PIN A0   // old:4
#define CSB_MUX_A A3  // old 0
#define CSB_MUX_B A4  // old 1
#define LED1_PIN 0
#define LED2_PIN 1
#define BUT1_PIN 14
#define BUT2_PIN 13

// #define DUE // Arduino DUE, early prototype MUX
#define MKR // used for all new MUXes with MKR ZERO controller

#ifdef MKR
const int A_SPI_SDIO = SDIO_PIN;  // SATA pin SI = 5, SO = 6
const int A_SPI_CSB  = CSB_MUX_A;  // SATA pin 2
const int A_SPI_SCLK = SCLK_PIN;  // SATA pin 3
const int B_SPI_SDIO = SDIO_PIN;  // SATA pin SI = 5, SO = 6
const int B_SPI_CSB  = CSB_MUX_B;  // SATA pin 2
const int B_SPI_SCLK = SCLK_PIN;  // SATA pin 3
#endif

#ifdef DUE
const int A_SPI_SDIO = 32;  // SATA pin SI = 5, SO = 6
const int A_SPI_CSB  = 30;  // SATA pin 2
const int A_SPI_SCLK = 31;  // SATA pin 3
const int B_SPI_SDIO = 44;  // SATA pin SI = 5, SO = 6
const int B_SPI_CSB  = 42;  // SATA pin 2
const int B_SPI_SCLK = 43;  // SATA pin 3
#endif

// Two RF_MUX objects, for the left and right board
RF_MUX::RF_MUX* a_mux = new RF_MUX::RF_MUX('1', A_SPI_SDIO, A_SPI_CSB, A_SPI_SCLK);
RF_MUX::RF_MUX* b_mux = new RF_MUX::RF_MUX('2', B_SPI_SDIO, B_SPI_CSB, B_SPI_SCLK);


// mux carrier v1 pins
const int LED1 =  LED1_PIN;
const int LED2 =  LED2_PIN;
const int BUTTON1 = BUT1_PIN;
const int BUTTON2 = BUT2_PIN;

// button states
int b1;
int b2;


HTTPD::httpd* webserver = new HTTPD::httpd(a_mux, b_mux);

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  delay(1000);
  Serial.print("initializing.\n");
  // initialize digital pin LED_BUILTIN as an output.
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  pinMode(BUTTON1, INPUT);
  pinMode(BUTTON2, INPUT);
  
  delay(1000);
  webserver->init();
  delay(1000);
  Serial.print("ready.\n");
}

void loop() {
  webserver->available();
  b1 = digitalRead(BUTTON1);
  b2 = digitalRead(BUTTON2);
  #ifdef MKR
    if (b1== HIGH) {
      webserver->advance(0); // pressing button advances state of MUX
      
    } else if (b2==HIGH) {
      webserver->advance(1);
    }
  #endif
  
  
}
